@@ -84,6 +84,7 @@ typedef struct PgStats PgStats;
 typedef union PgAddr PgAddr;
 typedef enum SocketState SocketState;
 typedef struct PktHdr PktHdr;
+typedef struct ScramState ScramState;
 
 extern int cf_sbuf_len;
 
@@ -110,32 +111,48 @@ extern int cf_sbuf_len;
 /* to avoid allocations will use static buffers */
 #define MAX_DBNAME	64
 #define MAX_USERNAME	64
-#define MAX_PASSWORD	128
+/* typical SCRAM-SHA-256 verifier takes at least 133 bytes */
+#define MAX_PASSWORD	160
+
+/*
+ * AUTH_* symbols are used for both protocol handling and
+ * configuration settings (auth_type, hba).  Some are only applicable
+ * to one or the other.
+ */
 
 /* no-auth modes */
 #define AUTH_ANY	-1 /* same as trust but without username check */
 #define AUTH_TRUST	AUTH_OK
 
-/* protocol codes */
+/* protocol codes in Authentication* 'R' messages from server */
 #define AUTH_OK		0
-#define AUTH_KRB	2
+#define AUTH_KRB4	1	/* not supported */
+#define AUTH_KRB5	2	/* not supported */
 #define AUTH_PLAIN	3
-#define AUTH_CRYPT	4
+#define AUTH_CRYPT	4	/* not supported */
 #define AUTH_MD5	5
-#define AUTH_CREDS	6
+#define AUTH_SCM_CREDS	6	/* not supported */
+#define AUTH_GSS	7	/* not supported */
+#define AUTH_GSS_CONT	8	/* not supported */
+#define AUTH_SSPI	9	/* not supported */
+#define AUTH_SASL	10
+#define AUTH_SASL_CONT	11
+#define AUTH_SASL_FIN	12
 
 /* internal codes */
-#define AUTH_CERT	7
-#define AUTH_PEER	8
-#define AUTH_HBA	9
-#define AUTH_REJECT	10
-#define AUTH_PAM	11
+#define AUTH_CERT	107
+#define AUTH_PEER	108
+#define AUTH_HBA	109
+#define AUTH_REJECT	110
+#define AUTH_PAM	111
+#define AUTH_SCRAM_SHA_256	112
 
 /* type codes for weird pkts */
 #define PKT_STARTUP_V2  0x20000
 #define PKT_STARTUP     0x30000
 #define PKT_CANCEL      80877102
 #define PKT_SSLREQ      80877103
+#define PKT_GSSENCREQ   80877104
 
 #define POOL_SESSION	0
 #define POOL_TX		1
@@ -373,6 +390,19 @@ struct PgSocket {
 		PgDatabase *db;			/* cache db while doing auth query */
 	};
 
+	struct ScramState {
+		char *client_nonce;
+		char *client_first_message_bare;
+		char *client_final_message_without_proof;
+		char *server_nonce;
+		char *server_first_message;
+		uint8_t	*SaltedPassword;
+		int iterations;
+		char *salt;	/* base64-encoded */
+		uint8_t StoredKey[32];	/* SHA256_DIGEST_LENGTH */
+		uint8_t ServerKey[32];
+	} scram_state;
+
 	VarCache vars;		/* state of interesting server parameters */
 
 	SBuf sbuf;		/* stream buffer, must be last */
@@ -448,6 +478,7 @@ extern char *cf_ignore_startup_params;
 extern char *cf_admin_users;
 extern char *cf_stats_users;
 extern int cf_stats_period;
+extern int cf_log_stats;
 
 extern int cf_pause_mode;
 extern int cf_shutdown;
