@@ -588,6 +588,8 @@ static bool parse_line(struct HBA *hba, struct TokParser *tp, int linenr, const
 		rule->rule_method = AUTH_PEER;
 	} else if (eat_kw(tp, "cert")) {
 		rule->rule_method = AUTH_CERT;
+	} else if (eat_kw(tp, "scram-sha-256")) {
+		rule->rule_method = AUTH_SCRAM_SHA_256;
 	} else {
 		log_warning("hba line %d: unsupported method: buf=%s", linenr, tp->buf);
 		goto failed;
