@@ -138,6 +138,7 @@ char *cf_jobname;
 char *cf_admin_users;
 char *cf_stats_users;
 int cf_stats_period;
+int cf_log_stats;
 
 int cf_log_connections;
 int cf_log_disconnections;
@@ -177,6 +178,7 @@ static const struct CfLookup auth_type_map[] = {
 #ifdef HAVE_PAM
 	{ "pam", AUTH_PAM },
 #endif
+	{ "scram-sha-256", AUTH_SCRAM_SHA_256 },
 	{ NULL }
 };
 
@@ -270,6 +272,7 @@ CF_ABS("verbose", CF_INT, cf_verbose, 0, NULL),
 CF_ABS("admin_users", CF_STR, cf_admin_users, 0, ""),
 CF_ABS("stats_users", CF_STR, cf_stats_users, 0, ""),
 CF_ABS("stats_period", CF_INT, cf_stats_period, 0, "60"),
+CF_ABS("log_stats", CF_INT, cf_log_stats, 0, "1"),
 CF_ABS("log_connections", CF_INT, cf_log_connections, 0, "1"),
 CF_ABS("log_disconnections", CF_INT, cf_log_disconnections, 0, "1"),
 CF_ABS("log_pooler_errors", CF_INT, cf_log_pooler_errors, 0, "1"),
