@@ -5,7 +5,6 @@
 
 cd $(dirname $0)
 
-export PATH=/usr/lib/postgresql/9.4/bin:$PATH
 export PGDATA=$PWD/pgdata
 export PGHOST=localhost
 export PGPORT=6667
@@ -16,15 +15,14 @@ BOUNCER_LOG=test.log
 BOUNCER_INI=test.ini
 BOUNCER_PID=test.pid
 BOUNCER_PORT=`sed -n '/^listen_port/s/listen_port.*=[^0-9]*//p' $BOUNCER_INI`
-BOUNCER_EXE="../pgbouncer"
+BOUNCER_EXE="$BOUNCER_EXE_PREFIX ../pgbouncer"
 
 LOGDIR=log
-NC_PORT=6668
 PG_PORT=6666
 PG_LOG=$LOGDIR/pg.log
 
 pgctl() {
-	pg_ctl -o "-p $PG_PORT" -D $PGDATA $@ >>$PG_LOG 2>&1
+	pg_ctl -w -o "-p $PG_PORT" -D $PGDATA $@ >>$PG_LOG 2>&1
 }
 
 ulimit -c unlimited
@@ -34,6 +32,21 @@ which initdb > /dev/null || {
 	exit 1
 }
 
+# System configuration checks
+SED_ERE_OP='-E'
+case `uname` in
+Linux)
+	SED_ERE_OP='-r'
+	;;
+esac
+
+pg_majorversion=$(initdb --version | sed -n $SED_ERE_OP 's/.* ([0-9]+).*/\1/p')
+if test $pg_majorversion -ge 10; then
+	pg_supports_scram=true
+else
+	pg_supports_scram=false
+fi
+
 # System configuration checks
 if ! grep -q "^\"${USER}\"" userlist.txt; then
 	cp userlist.txt userlist.txt.bak
@@ -59,16 +72,14 @@ if test -n "$CAN_SUDO"; then
 	esac
 fi
 
-# System configuration checks
-SED_ERE_OP='-E'
-case `uname` in
-Linux)
-	SED_ERE_OP='-r'
-	;;
-esac
-
 stopit() {
-	test -f "$1" && { kill `cat "$1"`; rm -f "$1"; }
+	local pid
+	if test -f "$1"; then
+		pid=`head -n1 "$1"`
+		kill $pid
+		while kill -0 $pid 2>/dev/null; do sleep 0.1; done
+		rm -f "$1"
+	fi
 }
 
 stopit test.pid
@@ -80,30 +91,61 @@ rm -rf $PGDATA
 
 if [ ! -d $PGDATA ]; then
 	mkdir $PGDATA
-	initdb >> $PG_LOG 2>&1
+	initdb --nosync >> $PG_LOG 2>&1
 	sed $SED_ERE_OP -i "/unix_socket_director/s:.*(unix_socket_director.*=).*:\\1 '/tmp':" pgdata/postgresql.conf
+	cat >>pgdata/postgresql.conf <<-EOF
+	log_connections = on
+	EOF
+	if $pg_supports_scram; then
+		cat >pgdata/pg_hba.conf <<-EOF
+		local  p6   all                scram-sha-256
+		host   p6   all  127.0.0.1/32  scram-sha-256
+		host   p6   all  ::1/128       scram-sha-256
+		EOF
+	else
+		cat >pgdata/pg_hba.conf </dev/null
+	fi
+	cat >>pgdata/pg_hba.conf <<-EOF
+	local  p4   all                password
+	host   p4   all  127.0.0.1/32  password
+	host   p4   all  ::1/128       password
+	local  p5   all                md5
+	host   p5   all  127.0.0.1/32  md5
+	host   p5   all  ::1/128       md5
+	local  all  all                trust
+	host   all  all  127.0.0.1/32  trust
+	host   all  all  ::1/128       trust
+	EOF
 fi
 
 pgctl start
-sleep 5
 
 echo "Creating databases"
 psql -X -p $PG_PORT -l | grep p0 > /dev/null || {
-	psql -X -o /dev/null -p $PG_PORT -c "create user bouncer" template1
-	createdb -p $PG_PORT p0
-	createdb -p $PG_PORT p1
-	createdb -p $PG_PORT p3
+	psql -X -o /dev/null -p $PG_PORT -c "create user bouncer" template1 || exit 1
+	for dbname in p0 p1 p3 p4 p5 p6; do
+		createdb -p $PG_PORT $dbname || exit 1
+	done
 }
 
 psql -X -p $PG_PORT -d p0 -c "select * from pg_user" | grep pswcheck > /dev/null || {
-	psql -X -o /dev/null -p $PG_PORT -c "create user pswcheck with superuser createdb password 'pgbouncer-check';" p0 || return 1
-	psql -X -o /dev/null -p $PG_PORT -c "create user someuser with password 'anypasswd';" p0 || return 1
+	echo "Creating users"
+	psql -X -o /dev/null -p $PG_PORT -c "create user pswcheck with superuser createdb password 'pgbouncer-check';" p0 || exit 1
+	psql -X -o /dev/null -p $PG_PORT -c "create user someuser with password 'anypasswd';" p0 || exit 1
+	if $pg_supports_scram; then
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'md5'; create user muser1 password 'foo';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'md5'; create user muser2 password 'wrong';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'md5'; create user puser1 password 'foo';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'md5'; create user puser2 password 'wrong';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = 'scram-sha-256'; create user scramuser1 password 'foo';" p0 || exit 1
+	else
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = on; create user muser1 password 'foo';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = on; create user muser2 password 'wrong';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = on; create user puser1 password 'foo';" p0 || exit 1
+		psql -X -o /dev/null -p $PG_PORT -c "set password_encryption = on; create user puser2 password 'wrong';" p0 || exit 1
+	fi
 }
 
-echo "Starting bouncer"
-$BOUNCER_EXE -d $BOUNCER_INI
-sleep 1
-
 #
 #  fw hacks
 #
@@ -166,7 +208,11 @@ admin() {
 runtest() {
 	local status
 
+	$BOUNCER_EXE -d $BOUNCER_INI
+	until psql -X -h /tmp -U pgbouncer -d pgbouncer -c "show version" 2>/dev/null 1>&2; do sleep 0.1; done
+
 	printf "`date` running $1 ... "
+	echo "# $1 begin" >>$BOUNCER_LOG
 	eval $1 >$LOGDIR/$1.log 2>&1
 	status=$?
 	if [ $status -eq 0 ]; then
@@ -175,13 +221,15 @@ runtest() {
 		echo "skipped"
 	else
 		echo "FAILED"
+		cat $LOGDIR/$1.log | sed 's/^/# /'
 	fi
 	date >> $LOGDIR/$1.log
 
 	# allow background processing to complete
 	wait
-	# start with fresh config
-	kill -HUP `cat $BOUNCER_PID`
+
+	stopit test.pid
+	echo "# $1 end" >>$BOUNCER_LOG
 
 	return $status
 }
@@ -229,8 +277,8 @@ test_idle_transaction_timeout() {
 	# test for GH issue #125
 	psql -X --set ON_ERROR_STOP=1 p0 <<-PSQL_EOF
 	begin;
-	select pg_sleep(1);
-	\! sleep 2
+	select pg_sleep(2);
+	\! sleep 1
 	select now();
 	PSQL_EOF
 	test $? -ne 0 && return 1
@@ -263,27 +311,23 @@ test_server_login_retry() {
 	return $rc
 }
 
-# server_connect_timeout - uses netcat to start dummy server
+# server_connect_timeout
 test_server_connect_timeout_establish() {
-	which nc >/dev/null || return 1
-	if nc -h 2>&1 | grep -q 'nc -l -p port'; then
-		# traditional or GNU style
-		set -- nc -l -p $NC_PORT
-	else
-		# BSD style
-		set -- nc -l $NC_PORT
-	fi
-	echo "$@"
-	"$@" >/dev/null &
-	sleep 2
+	psql -X -p $PG_PORT -c "alter system set pre_auth_delay to '60s'" p0
+	kill -HUP `head -n1 pgdata/postmaster.pid`
+	sleep 1
 
 	admin "set query_timeout=3"
 	admin "set server_connect_timeout=2"
-	psql -X -c "select now()" p2
+	psql -X -c "select now()" p0
 	# client will always see query_timeout, need to grep for connect timeout
 	grep "closing because: connect timeout" $BOUNCER_LOG
 	rc=$?
-	killall nc
+
+	rm -f pgdata/postgresql.auto.conf
+	kill -HUP `head -n1 pgdata/postmaster.pid`
+	sleep 1
+
 	return $rc
 }
 
@@ -462,6 +506,11 @@ test_database_restart() {
 
 	wait
 	psql -X -c "select now() as p0_after_restart" p0 || return 1
+
+	# connect to clear server_login_retry state
+	psql -X -c "select now() as p1_after_restart" p1
+
+	return 0
 }
 
 # test connect string change
@@ -526,7 +575,7 @@ test_fast_close() {
 test_wait_close() {
 	(
 		echo "select pg_backend_pid();"
-		sleep 2
+		sleep 3
 		echo "select pg_backend_pid();"
 		echo "\q"
 	) | psql -X -tAq -f- -d p3 &
@@ -534,9 +583,10 @@ test_wait_close() {
 	sleep 1
 	admin "reconnect p3"
 	admin "wait_close p3"
+	sleep 1  # give psql a moment to exit
 
-	# psql should no longer be running now.  (Without the wait it
-	# would still be running.)
+	# psql should no longer be running now.  (Without the
+	# wait_close it would still be running.)
 	kill -0 $psql_pid
 	psql_running=$?
 
@@ -570,6 +620,173 @@ test_auth_user() {
 	return 0
 }
 
+# test plain-text password authentication from PgBouncer to PostgreSQL server
+#
+# The PostgreSQL server no longer supports storing plain-text
+# passwords, so the server-side user actually uses md5 passwords in
+# this test case, but the communication is still in plain text.
+test_password_server() {
+	admin "set auth_type='trust'"
+
+	# good password from ini
+	psql -X -c "select 1" p4 || return 1
+	# bad password from ini
+	psql -X -c "select 2" p4x && return 1
+
+	# good password from auth_file
+	psql -X -c "select 1" p4y || return 1
+	# bad password from auth_file
+	psql -X -c "select 1" p4z && return 1
+
+	return 0
+}
+
+# test plain-text password authentication from client to PgBouncer
+test_password_client() {
+	admin "set auth_type='plain'"
+
+	# test with users that have a plain-text password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U puser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U puser2 -c "select 2" p1 && return 1
+
+	# test with users that have an md5 password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U muser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U muser2 -c "select 2" p1 && return 1
+
+	# test with users that have a SCRAM password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U scramuser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U scramuser2 -c "select 2" p1 && return 1
+
+	admin "set auth_type='trust'"
+
+	return 0
+}
+
+# test md5 authentication from PgBouncer to PostgreSQL server
+test_md5_server() {
+	admin "set auth_type='trust'"
+
+	# good password from ini
+	psql -X -c "select 1" p5 || return 1
+	# bad password from ini
+	psql -X -c "select 2" p5x && return 1
+
+	# good password from auth_file
+	psql -X -c "select 1" p5y || return 1
+	# bad password from auth_file
+	psql -X -c "select 1" p5z && return 1
+
+	return 0
+}
+
+# test md5 authentication from client to PgBouncer
+test_md5_client() {
+	admin "set auth_type='md5'"
+
+	# test with users that have a plain-text password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U puser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U puser2 -c "select 2" p1 && return 1
+
+	# test with users that have an md5 password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U muser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U muser2 -c "select 2" p1 && return 1
+
+	admin "set auth_type='trust'"
+
+	return 0
+}
+
+# test SCRAM authentication from PgBouncer to PostgreSQL server
+test_scram_server() {
+	$pg_supports_scram || return 77
+
+	admin "set auth_type='trust'"
+
+	# good password from ini
+	psql -X -c "select 1" p6 || return 1
+	# bad password from ini
+	psql -X -c "select 2" p6x && return 1
+
+	# good password from auth_file (fails: not supported with SCRAM)
+	psql -X -c "select 1" p6y && return 1
+	# bad password from auth_file
+	psql -X -c "select 1" p6z && return 1
+
+	return 0
+}
+
+# test SCRAM authentication from client to PgBouncer
+test_scram_client() {
+	$pg_supports_scram || return 77
+
+	admin "set auth_type='scram-sha-256'"
+
+	# test with users that have a plain-text password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U puser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U puser2 -c "select 2" p1 && return 1
+
+	# test with users that have an md5 password stored (all fail)
+
+	# good password
+	PGPASSWORD=foo psql -X -U muser1 -c "select 1" p1 && return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U muser2 -c "select 2" p1 && return 1
+
+	# test with users that have a SCRAM password stored
+
+	# good password
+	PGPASSWORD=foo psql -X -U scramuser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U scramuser2 -c "select 2" p1 && return 1
+
+	# SCRAM should also work when auth_type is "md5"
+	admin "set auth_type='md5'"
+
+	# good password
+	PGPASSWORD=foo psql -X -U scramuser1 -c "select 1" p1 || return 1
+	# bad password
+	PGPASSWORD=wrong psql -X -U scramuser2 -c "select 2" p1 && return 1
+
+	admin "set auth_type='trust'"
+
+	return 0
+}
+
+# test all the show commands
+#
+# This test right now just runs all the commands without checking the
+# output, which would be difficult.  This at least ensures the
+# commands don't completely die.  The output can be manually eyeballed
+# in the test log file.
+test_show() {
+	for what in clients config databases fds help lists pools servers sockets active_sockets stats stats_totals stats_averages users version totals mem dns_hosts dns_zones; do
+		    echo "=> show $what;"
+		    psql -X -h /tmp -U pgbouncer -d pgbouncer -c "show $what;" || return 1
+	done
+
+	psql -X -h /tmp -U pgbouncer -d pgbouncer -c "show bogus;" && return 1
+
+	return 0
+}
+
 testlist="
 test_server_login_retry
 test_auth_user
@@ -592,6 +809,13 @@ test_database_change
 test_reconnect
 test_fast_close
 test_wait_close
+test_password_server
+test_password_client
+test_md5_server
+test_md5_client
+test_scram_server
+test_scram_client
+test_show
 "
 
 if [ $# -gt 0 ]; then
